#!/bin/bash
# Copyleft 2010, Hajime Yoshimori <lugia.kun@gmail.com>

# Disable globbing
# PS4="(texmf-installer) "
set -f -e -m
## For Debug.
#set -x

if [ $# -lt 3 ]; then
  echo "Usage: $0 datadir mandir infodir"
  exit -1
fi

if [ "x${NUMJOBS}" == x ]; then
  NUMJOBS=1
fi

NUMJOBS=$((NUMJOBS+1))
function progress_bar {
  rate=$(($3*100/$2))
  oos=''
  t=$(($rate*30/100))
  while [ $t -gt 1 ]; do
    oos="${oos}="
    t=$((t-1))
  done
  if [ $t -gt 0 ]; then
    oos="${oos}>"
  fi
  printf '%-20.20s |%-30.30s| %7d / %7d (%3d%%)\n' "$1" "$oos" "$3" "$2" "$rate"
  return 0
}

target=$1
mandir=$2
infodir=$3

if [ x$target == x ]; then
  echo "Specify datadir."
  exit -1
fi

if [ x$mandir == x ]; then
  echo "Specify mandir."
  exit -1
fi

if [ x$infodir == x ]; then
  echo "Specify infodir."
  exit -1
fi

exclude_list_nl=`cat texmf-not-needed-dirs.lst`
exclude_file_nl=`cat texmf-not-needed-files.lst`
man_files_nl=`cat texmf-man-files.lst`
info_files_nl=`cat texmf-info-files.lst`
find_param="("
for e in $exclude_list_nl; do
  find_param="$find_param -path */$e -o"
done
find_param="$find_param -false ) -a -prune -o -not ("
man_rule="-not ("
info_rule="-not ("
for e in $exclude_file_nl; do
  find_param="$find_param -regex $e -o"
  man_rule="$man_rule -regex $e -o"
  info_rule="$info_rule -regex $e -o"
done
man_rule="$man_rule -false ) -a ("
info_rule="$info_rule -false ) -a ("
for e in $man_files_nl; do
  find_param="$find_param -regex $e -o"
  man_rule="$man_rule -regex $e -o"
done
for e in $info_files_nl; do
  find_param="$find_param -regex $e -o"
  info_rule="$info_rule -regex $e -o"
done
find_param="$find_param -false )"
man_rule="$man_rule -false )"
info_rule="$info_rule -false )"

texmf_file_lists="texmf-file-lists"
texmf_xfile_lists="texmf-xfile-lists"
texmf_man_file_lists="texmf-man-file-lists"
texmf_info_file_lists="texmf-info-file-lists"

if [ -e ${texmf_file_lists} ]; then
  echo "'${texmf_file_lists}' exists!"
  rm ${texmf_file_lists} 
fi

echo 'Collecting Information...'
dirs=(`find texmf{,-dist} ${find_param} -a -type d -a -print`)
if [ "x$dirs" == x ]; then
  echo 'No texmf directory found!'
  exit -1
fi
c=0
for d in ${dirs[@]}; do
  c=$((c+1))
  install -d $target/$d
  progress_bar 'Create Directory' ${#dirs[@]} $c
done
printf '\n'

find texmf{,-dist} ${find_param} \
                   -a -type f -a -not -perm -ugo+x \
                   -a -print \
  | sed -e 's,^\(.*/\)\([^/]*\)$,f:\1\2:\2,g' \
  | sort -k3 --field-separator=":" \
  >> ${texmf_file_lists}

find texmf{,-dist} ${find_param} \
                   -a -type f -a -perm -ugo+x \
                   -a -print \
  | sed -e 's,^\(.*/\)\([^/]*\)$,x:\1\2:\2,g' \
  | sort -k3 --field-separator=":" \
  >> ${texmf_file_lists}

find texmf{,-dist} ${man_rule} -a -print \
  | sed -e 's,^\(.*/\)\([^/]*\)$,m:\1\2:\2,g' \
  | sort -k3 --field-separator=":" \
  >> ${texmf_file_lists}

find texmf{,-dist} ${info_rule} -a -print \
  | sed -e 's,^\(.*/\)\([^/]*\)$,i:\1\2:\2,g' \
  | sort -k3 \
  >> ${texmf_file_lists}

c=0
total_files=`cat ${texmf_file_lists} | wc -l`
for f in `cat ${texmf_file_lists}`; do
  data=(`sed -e 's,:, ,g' << EOF
$f
EOF`)
  if [ ${data[0]} == "f" ]; then
    install -m 644 ${data[1]} $target/${data[1]} &
  elif [ ${data[0]} == "x" ]; then
    if file -b ${data[1]} | grep '\(texlua script\|shell script\|python\|perl\)' 1>/dev/null 2>/dev/null; then
      install -m 755 ${data[1]} $target/${data[1]} &
    else
      install -m 644 ${data[1]} $target/${data[1]} &
      data[0]='f'
    fi
  elif [ ${data[0]} == "m" ]; then
    suffix=`sed -e 's,^.*\.\([^\.]*\)$,\1,' <<EOF
${data[2]}
EOF`
    if [ ! -e $mandir/man$suffix ]; then
      install -d $mandir/man$suffix
    fi
    install -m 644 ${data[1]} $mandir/man$suffix/${data[2]} &
  else
    install -m 644 ${data[1]} $infodir/${data[2]} &
  fi
  c=$((c+1))
  progress_bar "${data[0]}:${data[2]}" $total_files $c
  fg ${NUMJOBS} 1>/dev/null 2>/dev/null || : 
done
printf '\n'
exit 0


