#!/bin/bash
# Copyleft 2010, Hajime Yoshimori <lugia.kun@gmail.com>

# Disable globbing
#PS4="(making files list) "
set -f -e
## For debug
#set -x

# It seems that we don't need this.
alias which 2>/dev/null && unalias which 2>/dev/null

if [ $# -lt 3 ]; then
  echo "Usage: $0 pkg-name real-texmf-installed-dir installed-texmf-dir -d dir -f file -p prune-dir -ed exclude-dir -ef exclude-file ..."
  echo
  echo "Type I   Confliction: The same directory as it is managed by others."
  echo "Type II  Confliction: The some parent directories of it are managed by others."
  echo "Type III Confliction: The following directories will be dominated by this package."
  exit -1
fi
at=($@)

if [ x$1 == x ]; then
  echo "Please specify package name."
  exit -1
fi
pkg_name="$1"
shift

if [ x$1 == x ]; then
  echo "Please specify real texmf directory (ex. /usr/share)."
  exit -1
fi
real_dir="$1"
shift

if [ x$1 == x ]; then
  echo "Please specify installed texmf directory."
  exit -1
fi
inst_dir="$1"

if [ ! -e $inst_dir ]; then
  echo "$inst_dir: No such file or directory"
  exit -1
fi

params=("$@")
append=0
dirs="("
files="("
pfiles="("
pdirs="("
exdirs="("
exfiles="("
while shift; do
  if [ "x$1" == "x" ]; then
    continue
  fi
  case $1 in
  -d)
    dirs="$dirs -path */$2 -o"
    ;;
  -f)
    files="$files -regex $2 -o"
    ;;
  -p)
    pdirs="$pdirs -path */$2 -o"
    ;;
 -pf)
    pfiles="$pfiles -regex $2 -o"
    ;;
 -ed)
    exdirs="$exdirs -path */$2 -o"
    ;;
  -ef)
    exfiles="$exfiles -regex $2 -o"
    ;;
  *)
    echo "Parse Error: $1"
    echo "Usage: $0 pkg-name real-texmf-installed-dir installed-texmf-dir -d dir -f file -p prune-dir -pf prune-pattern -ed exclude-dir -ef exclude-file ..."
    exit -1
    ;;
  esac
  shift
done
dirs="$dirs -false ) -a -prune"
files="$files -false )"
pdirs="$pdirs -false )"
pfiles="$pfiles -false )"
exdirs="$exdirs -false ) -a -prune"
exfiles="$exfiles -false )"

outname="${pkg_name}-files"
outname=`sed -e 's,|,,g' <<EOF
${outname}
EOF`
if [ "x$outname" == "x" -o -e $outname ]; then
  echo "Specify another name: $outname: File exists"
  exit -1
fi

if [ -e ${outname}~ ]; then
  rm -v ${outname}~
fi

running_dir=`pwd`
pushd $inst_dir 1>/dev/null

find texmf{,-dist} ${exdirs} \
                -o ${dirs} -a "(" -not ${exfiles} ")" \
                -a -print \
                | sed -e 's,^,#dir '"$real_dir"'/,g' \
                >> ${running_dir}/${outname}~

find texmf{,-dist} ${exdirs} \
                -o ${files} -a "(" -not ${exfiles} ")" \
                -a -print \
                | sed -e 's,^,#dir '"$real_dir"'/,g' \
                >> ${running_dir}/${outname}~

find texmf{,-dist} "(" ${exdirs} -a ${dirs} ")" \
                   -o ${pdirs} -a "(" -not ${exfiles} ")" \
                   -a -type d -a -print \
                | sed -e 's,^,%dir '"$real_dir"'/,g' \
                >> ${running_dir}/${outname}~

find texmf{,-dist} "(" ${exdirs} -a ${dirs} ")" \
                -o ${pfiles} -a "(" -not ${exfiles} ")" \
                -a -type d -a -print \
                | sed -e 's,^,%dir '"$real_dir"'/,g' \
                >> ${running_dir}/${outname}~

popd 1>/dev/null

sort -k2 ${outname}~ | uniq --skip-fields=1 | sort - \
     | sed -e '1 i \
# '"`echo "$0" "${at[@]}" | sed -e 's,%,%%,g'`" -e '1 i \
%defattr(-,root,root)' -e 's,^#dir ,,g' \
      >> ${outname}

# enable globbing
set +f +x

if [ -e ${outname}.sed ]; then
  rm -v ${outname}.sed
fi

echo "===== Conflicts and Files Inspection Result ====="
echo "=== `printf "%41s" "for $pkg_name"` ==="
echo "===================== BEGIN ====================="

if [ -e 'generated-file-list' ]; then
  files=(`cat 'generated-file-list' | sed -e '/^'"${outname}"'$/ d' | sort -`)
else
  files=()
fi
if [ ${#files[@]} == 0 ]; then
  cat ${outname}
  echo "====================== END ======================"

  echo
  echo ${outname} > 'generated-file-list'
  rm ${outname}~
  exit 0
fi

(
  if [ ${#files[@]} == 1 ]; then
	cat "${files[@]}" | sed -e "s,^,${files[@]}:,"
  else
	grep '.*' "${files[@]}"
  fi
) \
    | sed -e 's,:, ,g' -e '\c^#.*c d' -e '\c.*%defattr.*c d' -e '\c.*%dir.*c d' \
    | sort - -k2 \
    | sed -e 's,\([^ ]*\) \(.*\),\\|^\\(%dir \\)*\2/| i \\\
!! conflicts with \1 !! (Type II),g' \
    | sed -f - ${outname} \
    | tee ${outname}~

grep '^\(/\|%dir\).*' "${files[@]}" "${outname}" \
  | sed -e 's,:\([^%]\),:#dir \1,g' \
  | sort - -k2 \
  | uniq -D --skip-fields=1 \
  | grep -v '^${outname}:' \
  | sed -e 's,:, ,g' -e 's,\([^ ]*\) [^ ]* \(.*\),!! conflicts with \1 !! (Type I)\n\2,g' \
  | tee -a ${outname}~

sed -e '\:^/.*: ! d' \
    -e 's,\(.*\),\\|\1/| s|\\([^ ]*\\) \\(.*\\)|!! conflicts with \\1 !! (Type III)\\n\\2|g,g' ${outname} \
  > ${outname}.sed

(
  if [ ${#files[@]} == 1 ]; then
    grep '^\(/\|%dir\).*' "${files[@]}" | sed -e "s,^,${files[0]}:,"
  else
    grep '^\(/\|%dir\).*' "${files[@]}"
  fi
) \
  | sed -e 's,%dir ,,g' -e 's,:, ,g' \
  | sort - -k2 \
  | sed -f ${outname}.sed - \
  | grep 'conflicts with' -A1 \
  | sed -e '\c^--$c d' \
  | tee -a ${outname}~

echo "====================== END ======================"

if grep "conflicts with" ${outname}~ 1>/dev/null 2>/dev/null; then
  rm -v ${outname}{,~,.sed}
  exit -1
fi

echo
sed -i -e '\|^'"${outname}"'$| d' 'generated-file-list'
cat 'generated-file-list' - <<EOF | sort - > 'generated-file-list~'
${outname}
EOF
mv 'generated-file-list~' 'generated-file-list'
rm ${outname}{~,.sed}
exit 0
